'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var isPlainObject = _interopDefault(require('is-plain-object'));

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function formatKey(value) {
  const numValue = +value;
  return isNaN(numValue) ? value : numValue;
}

function isValidEnumValue(value) {
  return typeof value === 'number' || typeof value === 'string';
}

function isNum(value) {
  return typeof value === 'number';
}

function isString(value) {
  return typeof value === 'string';
}

function makeObject(keys, values) {
  const obj = {};
  let i = 0;
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = keys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      const key = _step.value;
      obj[key] = values[i];
      i += 1;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return obj;
}

function getKeyValueObject(input) {
  const obj = {};
  let keys = [];
  let values = [];
  let target = {};

  if (typeof input === 'object' && input !== null && isPlainObject(input)) {
    keys = Object.keys(input);
    values = Object.values(input);
    target = input;
  } else if (Array.isArray(input)) {
    keys = Array.from(new Set(input));
    values = keys;
    target = makeObject(keys, input);
  } else if (input instanceof Set && input.size > 0) {
    keys = Array.from(input);
    values = keys;
    target = makeObject(keys, values);
  } else if (input instanceof Map && input.size > 0) {
    keys = Array.from(input.keys());
    values = Array.from(input.values());
    target = makeObject(keys, values);
  }

  if (keys.every(isValidEnumValue) === false || values.every(isValidEnumValue) === false) {
    return obj;
  }

  keys = keys.map(formatKey);
  const stringKeys = keys.filter(isString);
  const numKeys = keys.filter(isNum);

  if (stringKeys.length !== numKeys.length || keys.length / 2 !== stringKeys.length) {
    return target;
  }

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = stringKeys[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      const key = _step2.value;
      obj[key] = target[key];
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return obj;
}

const EnumUtils = {
  entries(e) {
    return Object.entries(getKeyValueObject(e));
  },

  keys(e) {
    return Array.isArray(e) ? e : Object.keys(getKeyValueObject(e));
  },

  keysByValue(e) {
    const obj = getKeyValueObject(e);
    const keysByValue = {};

    var _arr = Object.entries(obj);

    for (var _i = 0; _i < _arr.length; _i++) {
      const _arr$_i = _slicedToArray(_arr[_i], 2),
            key = _arr$_i[0],
            keyValue = _arr$_i[1];

      keysByValue[keyValue] = key;
    }

    return keysByValue;
  },

  values(e) {
    return Array.isArray(e) ? e : Object.values(getKeyValueObject(e));
  },

  valuesByKey(input) {
    return getKeyValueObject(input);
  }

};

exports.EnumUtils = EnumUtils;
exports.default = EnumUtils;
