"use strict";

import isPlainObject from 'is-plain-object';

function formatKey(value) {
  const numValue = +value;
  return isNaN(numValue) ? value : numValue;
}

function isValidEnumValue(value) {
  return typeof value === 'number' || typeof value === 'string';
}

function isNum(value) {
  return typeof value === 'number';
}

function isString(value) {
  return typeof value === 'string';
}

function makeObject(keys, values) {
  const obj = {};
  let i = 0;

  for (const key of keys) {
    obj[key] = values[i];
    i += 1;
  }

  return obj;
}

function getKeyValueObject(input) {
  const obj = {};
  let keys = [];
  let values = [];
  let target = {};

  if (typeof input === 'object' && input !== null && isPlainObject(input)) {
    keys = Object.keys(input);
    values = Object.values(input);
    target = input;
  } else if (Array.isArray(input)) {
    keys = Array.from(new Set(input));
    values = keys;
    target = makeObject(keys, input);
  } else if (input instanceof Set && input.size > 0) {
    keys = Array.from(input);
    values = keys;
    target = makeObject(keys, values);
  } else if (input instanceof Map && input.size > 0) {
    keys = Array.from(input.keys());
    values = Array.from(input.values());
    target = makeObject(keys, values);
  }

  if (keys.every(isValidEnumValue) === false || values.every(isValidEnumValue) === false) {
    return obj;
  }

  keys = keys.map(formatKey);
  const stringKeys = keys.filter(isString);
  const numKeys = keys.filter(isNum);

  if (stringKeys.length !== numKeys.length || keys.length / 2 !== stringKeys.length) {
    return target;
  }

  for (const key of stringKeys) {
    obj[key] = target[key];
  }

  return obj;
}

export const EnumUtils = {
  entries(e) {
    return Object.entries(getKeyValueObject(e));
  },

  keys(e) {
    return Array.isArray(e) ? e : Object.keys(getKeyValueObject(e));
  },

  keysByValue(e) {
    const obj = getKeyValueObject(e);
    const keysByValue = {};

    for (const [key, keyValue] of Object.entries(obj)) {
      keysByValue[keyValue] = key;
    }

    return keysByValue;
  },

  values(e) {
    return Array.isArray(e) ? e : Object.values(getKeyValueObject(e));
  },

  valuesByKey(input) {
    return getKeyValueObject(input);
  }

};
export default EnumUtils;